// $Id: ctf2xml2.cc,v 1.1 2002/03/24 21:17:28 martin Exp $
// Martin Vidner, 2001
// ctf2xml is free software with no express or implied warranty.
// You are free to copy and modify it but you must keep the copyright
// and permissions notices intact.

#include <ccdocctf.h>
#include <ccdocstringrep.h>

CCcDocCtf ctf;

static const char *number(unsigned long int i) {
  static char buf[100];
  sprintf(buf, "%lu", i);
  return buf;
}

static void sxmlSetProp(FILE *f, const char *prop, const char *val) {
  const char *p;
  //TODO: encodings
  fprintf(f, " %s=\"", prop);
  for (p = val; *p != '\0'; ++p) {
    switch (*p) {
    case '&':
      fprintf(f, "&amp;");
      break;
    case '"':
      fprintf(f, "&quot;");
      break;
    case '<':
      fprintf(f, "&lt;");
      break;
    case '>':
      fprintf(f, "&gt;");
      break;
    default:
      fputc(*p, f);
      break;
    }
  }
  fprintf(f, "\"");
}

static void WriteTree(CCcDocParserNode* node, void* arg) {
  FILE *f = (FILE *) arg;

  fprintf(f, "<%s", node->GetTypeName());

  //sxmlSetProp(f, "l", node->GetLongTypeName());
  sxmlSetProp(f, "scope", node->GetScopeName());
  //sxmlSetProp(f, "xref", number(node->GetRecId()));
  //sxmlSetProp(f, "level", number(node->GetLevel()));
  //sxmlSetProp(f, "offset", number(node->GetOffset()));
  sxmlSetProp(f, "line", number(node->GetLineno()));
  //sxmlSetProp(f, "items", number(node->GetNumItems()));
  sxmlSetProp(f, "name", CCcDocStringRep::GetString(node->GetNameId()));

  fprintf(f, ">\n");

  // Now write out the data.
  unsigned int items = node->GetNumItems();
  if (items == 0) 
    fprintf(f, "<tokens/>\n");
  else {
    fprintf(f, "<tokens>\n");
    for(unsigned int i = 0; i < items; i++) {
      long id = node->Get(i);
      char *dt;
      switch(node->GetDataType(i)) {
      case CCcDocParserNode::DATA_ID:
	dt = "i";
	break;
      case CCcDocParserNode::DATA_RESERVED:
	dt = "r";
	break;
      case CCcDocParserNode::DATA_KEYWORD:
	dt = "k";
	break;
      default:
	dt = "?";
	break;
      }
      fprintf(f, "<tok");
      sxmlSetProp(f, "type", dt);
      sxmlSetProp(f, "value", CCcDocStringRep::GetString(id));
      fprintf(f, "/>\n");
    }
    fprintf(f, "</tokens>\n");
  }

  // Recurse
  for(CCcDocParserNode* subnode = node->GetFirst(); subnode ; subnode = node->GetNext()) 
    WriteTree(subnode, f);
  
  fprintf(f, "</%s>\n", node->GetTypeName());
}


int main(int argc, char **argv) {
  if (argc != 3) {
    fprintf(stderr, "usage: %s ctf-file xml-file\n", argv[0]);
    exit(1);
  }
     
  ctf.Read(argv[1]);

  FILE *f;
  if (!strcmp(argv[2], "-"))
    f = stdout;
  else 
    f = fopen(argv[2], "w");

  fprintf(f, "<?xml version=\"1.0\"?>\n");
  fprintf(f, "<!DOCTYPE ccdoc SYSTEM \"ccdoc.dtd\">\n");
  fprintf(f, "<ccdoc>\n");

  ctf.WalkRoots(WriteTree, f);

  fprintf(f, "</ccdoc>\n");
  fclose(f);
  return 0;
}
